/*
Rocket Flight Logger ver 1.4
 Copyright Boris du Reau 2012-2013
 
 The following is a datalogger for logging rocket flight.
 So far it can log the rocket altitude during the flight.
 
 
 This is using a BMP085 presure sensor and an Atmega 328
 The following record the flight in an EEPROM
 
 
 For the BMP085 pressure sensor
 Connect VCC of the BMP085 sensor to 5.0V! make sure that you are using the 5V sensor (GY-65 model)
 Connect GND to Ground
 Connect SCL to i2c clock - on 328 Arduino Uno/Duemilanove/etc thats Analog 5
 Connect SDA to i2c data - on 328 Arduino Uno/Duemilanove/etc thats Analog 4
 EOC is not used, it signifies an end of conversion
 
 For the EEPROM 24LC512-I/P - SERIAL EEPROM 512K,24LC512, DIP8
 
 Number Name ConnectTo     Note
 1       a0     GND
 2       a1     GND
 3       a2     GND
 4       GND    GND
 5       SDA    SDA         2kOhm PullUp
 6       SCL    SCL
 7       WP     GND
 8       VCC    (+3v3 ... +5V0)
 
 The time to write in the EEPROM is 10ms
 
 
 Major changes on version 1.4
 Altimeter configuration has been softcoded
 */

#define MAJOR_VERSION 1
#define MINOR_VERSION 5

#include <Wire.h> //I2C library
#include <Adafruit_BMP085.h>
//used for writing in the microcontroleur lib
#include <EEPROM.h>
//////////////////////////////////////////////////////////////////////
// Global variables
//////////////////////////////////////////////////////////////////////
#define CONFIG_START 32


struct ConfigStruct {
  /*char altiName[9]; 
   int  majorVersion;
   int  minorVersion;*/
  int unit;
  int beepingMode;
  int outPut1;
  int outPut2;
  int outPut3;
  int  mainAltitude;
  int superSonicYesNo;
  int outPut1Delay;
  int outPut2Delay;
  int outPut3Delay;
  int beepingFrequency;
  int  cksum;  
};
ConfigStruct config;

int mode = 0; //0 = read; 1 = write;
Adafruit_BMP085 bmp;
long endAddress = 65536;

//ground level altitude
long initialAltitude;
long liftoffAltitude;
long lastAltitude;
//current altitude
long currAltitude;
//Apogee altitude
long apogeeAltitude;
long mainAltitude;
boolean liftOff= false;
unsigned long initialTime;
boolean NoBeep=false;
boolean FastReading = false;

long currentMemaddress;
long startMemaddress;
long currentFileNbr = 1;
//Our drogue has been ejected i.e: apogee has been detected
boolean apogeeHasFired =false;
boolean mainHasFired=false;
boolean noContinuity = false;
//nbr of measures to do so that we are sure that apogee has been reached 
unsigned long measures=5;
unsigned long mainDeployAltitude;

// pin used by the jumpers
const int pinAltitude1 = 8;
const int pinAltitude2 = 7;

//pyro out 1
const int pyroOut1 = 9;
int pinApogee = 9;
//pyro out 2
const int pyroOut2= 13;
int pinMain = 13;
//pyro out 3
const int pyroOut3=17;
int pinOut3 =17;

//soft configuration
boolean softConfigValid =false;

const int pinApogeeContinuity = 10;
const int pinMainContinuity = 11;
const int pinOut3Continuity = 16;

const int pinSpeaker = 12;

//Kalman Variables
float f_1=1.00000;  //cast as float
float kalman_x;
float kalman_x_last;
float kalman_p;
float kalman_p_last;
float kalman_k;
float kalman_q;
float kalman_r;
float kalman_x_temp;
float kalman_p_temp;
//end of Kalman Variables
//#define NBR_MEASURE_APOGEE 5
float FEET_IN_METER =1;

typedef struct fileaddresses
{
  int startaddress;
  int endaddress;
};

fileaddresses fileA[25];

//================================================================
// EEPROM functions 
//================================================================
void i2c_eeprom_write_byte( int deviceaddress, unsigned int eeaddress, byte data ) {
  int rdata = data;
  int writeDelay=10;
  Wire.beginTransmission(deviceaddress);
  Wire.write((int)(eeaddress >> 8)); // MSB
  Wire.write((int)(eeaddress & 0xFF)); // LSB
  Wire.write(rdata);
  Wire.endTransmission();
  delay(writeDelay);
}

byte i2c_eeprom_read_byte( int deviceaddress, unsigned int eeaddress ) {
  byte rdata = 0xFF;
  int readDelay=5;
  Wire.beginTransmission(deviceaddress);
  Wire.write((int)(eeaddress >> 8)); // MSB
  Wire.write((int)(eeaddress & 0xFF)); // LSB
  Wire.endTransmission();
  Wire.requestFrom(deviceaddress,1);
  if (Wire.available()) rdata = Wire.read();
  return rdata;
}

void i2c_eeprom_erase_fileList()
{
  Serial.println(F("Start erasing files"));
  for (long i = 0; i < 101; i++)
  {
    i2c_eeprom_write_byte( 0x50, i, 0x00 );
  }
  currentFileNbr = 1;
  Serial.println(F("Done erasing file"));
}

//================================================================
// EEPROM Flight management functions 
//================================================================
int getFlightList()
{
  //get address where we are allowed to record
  //read the table to see what has been recorded so far

  int i = 0;
  //long 
  currentFileNbr = 1;
  byte startaddressMSB = B00000000;
  byte startaddressLSB = B00000000;
  byte endaddressMSB = B00000000;
  byte endaddressLSB = B00000000;
  boolean finish = false;

  while (i < 101 && finish == false)
  {
    int startaddress;
    int endaddress;
    startaddressMSB = i2c_eeprom_read_byte(0x50, i);
    i++;
    startaddressLSB = i2c_eeprom_read_byte(0x50, i);
    i++;
    startaddress = (startaddressMSB << 8) | startaddressLSB;

    endaddressMSB = i2c_eeprom_read_byte(0x50, i);
    i++;
    endaddressLSB = i2c_eeprom_read_byte(0x50, i);
    i++;
    endaddress = (endaddressMSB << 8) | endaddressLSB;

    fileA[currentFileNbr].startaddress = startaddress;
    fileA[currentFileNbr].endaddress = endaddress;

    currentFileNbr++;
    if (startaddress == 0) 
    {
      currentFileNbr = currentFileNbr -1;
      currentMemaddress =fileA[currentFileNbr-1].endaddress +1;
      finish = true;
    } 
  } 
  if (currentMemaddress == 1)
    currentMemaddress=101;

  startMemaddress = currentMemaddress; 
  //return the number of flight
  return (currentFileNbr-1);
}

boolean saveFlightA(int flightNbr, long startAdress, long endAdress)
{

  byte startaddressMSB = B00000000;
  byte startaddressLSB = B00000000;
  byte endaddressMSB = B00000000;
  byte endaddressLSB = B00000000;

  if (flightNbr < 26)
  {
    //save start address
    startaddressMSB = startMemaddress >> 8;
    i2c_eeprom_write_byte(0x50, (currentFileNbr*4)-4, startaddressMSB);

    startaddressLSB = startMemaddress;
    i2c_eeprom_write_byte(0x50, (currentFileNbr*4)-3, startaddressLSB); 
    endaddressMSB = (currentMemaddress -1) >> 8;
    i2c_eeprom_write_byte(0x50, (currentFileNbr*4)-2, endaddressMSB);
    endaddressLSB = (currentMemaddress -1);
    i2c_eeprom_write_byte(0x50, (currentFileNbr*4)-1, endaddressLSB);
    return true;
  }
  else
    return false;

}

//================================================================
// Start program
//================================================================
void setup()
{
  int val = 0;     // variable to store the read value
  int val1 = 0;     // variable to store the read value

  //pyro out 1
  pinApogee = pyroOut1;
  //pyro out 2
  pinMain= pyroOut2;
  //pyro out 3
  pinOut3= pyroOut3;


  // Read altimeter softcoded configuration
  softConfigValid = readAltiConfig();
  // check if configuration is valid 
  if (!softConfigValid)
  {
    //default values
    defaultConfig();
    int i;
    for( i=0; i<sizeof(config); i++ ) {
      EEPROM.write(CONFIG_START+i, *((char*)&config + i));
    }
  }
  //Assign Pyro output to main apogee or airstart
  getOutPin(config.outPut1);
  getOutPin(config.outPut2);
  getOutPin(config.outPut2);

  //set main altitude (if in feet convert to metrics)
  if(config.unit ==0)
    FEET_IN_METER =1;
  else
    FEET_IN_METER =3.28084 ;

  mainDeployAltitude= int(config.mainAltitude /FEET_IN_METER); 
  //beepFrequency


  //init Kalman filter
  KalmanInit();

  // initialise the connection
  Wire.begin(); 

  //You can change the baud rate here 
  //and change it to 57600, 115200 etc..
  Serial.begin(9600);

  pinMode(A0, INPUT);

  //Presure Sensor Initialisation
  bmp.begin( BMP085_STANDARD);
  //our drogue has not been fired
  apogeeHasFired=false;
  mainHasFired=false;

  Serial.print(F("Start program\n"));

  //Initialise the output pin
  pinMode(pinApogee, OUTPUT);
  pinMode(pinMain, OUTPUT);
  pinMode(pinOut3, OUTPUT);
  pinMode(pinSpeaker, OUTPUT);

  pinMode(pinAltitude1, INPUT);
  pinMode(pinAltitude2, INPUT);

  pinMode(pinApogeeContinuity, INPUT);
  pinMode(pinMainContinuity, INPUT);
  pinMode(pinOut3Continuity, INPUT);

  //Make sure that the output are turned off
  digitalWrite(pinApogee, LOW);
  digitalWrite(pinMain, LOW);
  digitalWrite(pinSpeaker, LOW);

  //initialisation give the version of the altimeter
  //One long beep per major number and One short beep per minor revision
  //For example version 1.2 would be one long beep and 2 short beep
  beepAltiVersion(MAJOR_VERSION,MINOR_VERSION);

  //number of measures to do to detect Apogee
  measures = 5; //NBR_MEASURE_APOGEE;

  if (!softConfigValid)
  {
    //initialise the deployement altitude for the main 
    mainDeployAltitude = 100;

    // On the Alti duo when you close the jumper you set it to 1
    // val is the left jumper and val1 is the right jumper
    //as of version 1.4 only use the jumper if no valid softconfiguration

    val = digitalRead(pinAltitude1); 
    val1 = digitalRead(pinAltitude2);  
    if(val == 0 && val1 ==0)
    {
      mainDeployAltitude = 50;
    }
    if(val == 0 && val1 ==1)
    {
      mainDeployAltitude = 100;
    }
    if(val == 1 && val1 ==0)
    {
      mainDeployAltitude = 150;
    }
    if(val == 1 && val1 ==1)
    {
      mainDeployAltitude = 200;
    }
  }
  // let's do some dummy altitude reading
  // to initialise the Kalman filter
  for (int i=0; i<50; i++){
    KalmanCalc(bmp.readAltitude());
  }

  //let's read the launch site altitude
  long sum = 0;
  for (int i=0; i<10; i++){
    sum += KalmanCalc(bmp.readAltitude());
    delay(50); 
  }
  initialAltitude = (sum / 10.0);
  lastAltitude = 0;//initialAltitude; 
  liftoffAltitude = 20;

  getFlightList();

}

void printFlightList()
{
  int startaddress;
  int endaddress;
  int flightNbr= 1;
  boolean exit=false;

  getFlightList();
  while(exit == false)
  {
    startaddress = fileA[flightNbr].startaddress;
    endaddress = fileA[flightNbr].endaddress;
    if (startaddress == 0) 
      exit = true;
    else
    {
      Serial.println("Flight Nbr: " + String(flightNbr) + " Start address: " + String(startaddress) + " End address: " + String(endaddress) + "\n");  
      flightNbr ++;
    }
  }
}

//================================================================
// Main loop which call the menu
//================================================================
void loop()
{
  MainMenu();
}  

void printFlight(int flightNbr)
{

  byte startaddressMSB = B00000000;
  byte startaddressLSB = B00000000;
  byte endaddressMSB = B00000000;
  byte endaddressLSB = B00000000;
  int startaddress;
  int endaddress;

  startaddressMSB = i2c_eeprom_read_byte(0x50, (flightNbr * 4)-4);

  startaddressLSB = i2c_eeprom_read_byte(0x50, (flightNbr * 4)-3);

  startaddress = (startaddressMSB << 8) | startaddressLSB;

  endaddressMSB = i2c_eeprom_read_byte(0x50, (flightNbr * 4)-2);

  endaddressLSB = i2c_eeprom_read_byte(0x50, (flightNbr * 4)-1);

  endaddress = (endaddressMSB << 8) | endaddressLSB;
  if (startaddress>100)
  {
    int i = startaddress;
    unsigned long currentTime=0;
    Serial.println("StartFlight;" );
    while(i < (endaddress +1))
    {

      byte currentTimeMSB = B00000000;
      byte currentTimeLSB = B00000000;
      byte currAltitudeMSB = B00000000;
      byte currAltitudeLSB = B00000000;

      currentTimeMSB = i2c_eeprom_read_byte(0x50, i);
      i++;
      currentTimeLSB = i2c_eeprom_read_byte(0x50, i);
      i++;

      currAltitudeMSB = i2c_eeprom_read_byte(0x50, i);
      i++;
      currAltitudeLSB = i2c_eeprom_read_byte(0x50, i);
      i++;
      currentTime = currentTime + ((currentTimeMSB << 8) | currentTimeLSB);
      //Serial.println(String(currentTime) + ","+ String((currAltitudeMSB << 8) | currAltitudeLSB) + "\n");
      Serial.println(String(currentTime) + ","+ String((currAltitudeMSB << 8) | currAltitudeLSB) );
      //Serial.println(String((currentTimeMSB << 8) | currentTimeLSB) + ","+ String((currAltitudeMSB << 8) | currAltitudeLSB) + "\n");
    }
    Serial.println("EndFlight;" );
  }
  else
    Serial.println(F("No such flight\n"));
}

void printFlightData(int flightNbr)
{

  byte startaddressMSB = B00000000;
  byte startaddressLSB = B00000000;
  byte endaddressMSB = B00000000;
  byte endaddressLSB = B00000000;
  int startaddress;
  int endaddress;

  startaddressMSB = i2c_eeprom_read_byte(0x50, (flightNbr * 4)-4);

  startaddressLSB = i2c_eeprom_read_byte(0x50, (flightNbr * 4)-3);

  startaddress = (startaddressMSB << 8) | startaddressLSB;

  endaddressMSB = i2c_eeprom_read_byte(0x50, (flightNbr * 4)-2);

  endaddressLSB = i2c_eeprom_read_byte(0x50, (flightNbr * 4)-1);

  endaddress = (endaddressMSB << 8) | endaddressLSB;
  if (startaddress>100)
  {
    int i = startaddress;
    unsigned long currentTime=0;
    //Serial.println("StartFlight;" );
    while(i < (endaddress +1))
    {

      byte currentTimeMSB = B00000000;
      byte currentTimeLSB = B00000000;
      byte currAltitudeMSB = B00000000;
      byte currAltitudeLSB = B00000000;

      currentTimeMSB = i2c_eeprom_read_byte(0x50, i);
      i++;
      currentTimeLSB = i2c_eeprom_read_byte(0x50, i);
      i++;

      currAltitudeMSB = i2c_eeprom_read_byte(0x50, i);
      i++;
      currAltitudeLSB = i2c_eeprom_read_byte(0x50, i);
      i++;
      currentTime = currentTime + ((currentTimeMSB << 8) | currentTimeLSB);
      //Serial.println(String(currentTime) + ","+ String((currAltitudeMSB << 8) | currAltitudeLSB) + "\n");
      Serial.println("$" + String("data,") +String(flightNbr) +","+String(currentTime) + ","+ String((currAltitudeMSB << 8) | currAltitudeLSB) +";");
      //Serial.println(String((currentTimeMSB << 8) | currentTimeLSB) + ","+ String((currAltitudeMSB << 8) | currAltitudeLSB) + "\n");
    }
    //Serial.println("EndFlight;" );
  }
  //else
  //Serial.println(F("No such flight\n"));
}

void recordAltitude()
{
  boolean exit=false;
  boolean apogeeReadyToFire = false;
  boolean mainReadyToFire = false;
  unsigned long apogeeStartTime =0;
  unsigned long mainStartTime =0;
  

  while (exit == false) 
  {

    //read current altitude
    currAltitude = (KalmanCalc(bmp.readAltitude())- initialAltitude);
    if (( currAltitude > liftoffAltitude) == true && liftOff == false)
    {
      liftOff = true;
      // save the time
      initialTime =millis();
    }  
    if(liftOff)
    {
      Serial.println(F("we have lift off\n"));
      //Save start address
      fileA[currentFileNbr].startaddress = startMemaddress;

      unsigned long prevTime=0;
      // loop until we have reach an altitude of 3 meter
      while(currAltitude > 3)
      {
        unsigned long currentTime;
        unsigned long diffTime;

        byte currentTimeMSB = B00000000;
        byte currentTimeLSB = B00000000;

        byte currAltitudeMSB = B00000000;
        byte currAltitudeLSB = B00000000;

        currAltitude = (KalmanCalc(bmp.readAltitude())- initialAltitude);

        currentTime = millis()- initialTime;
        diffTime = currentTime - prevTime;
        prevTime = currentTime;
        //currentTimeLSB = currentTime;
        //currentTimeMSB = currentTime >> 8;
        currentTimeLSB = diffTime;
        currentTimeMSB = diffTime >> 8;
        i2c_eeprom_write_byte(0x50, currentMemaddress, currentTimeMSB);
        currentMemaddress++;
        i2c_eeprom_write_byte(0x50, currentMemaddress, currentTimeLSB);
        currentMemaddress++;

        currAltitudeLSB = currAltitude;
        currAltitudeMSB = currAltitude >> 8;
        i2c_eeprom_write_byte(0x50, currentMemaddress, currAltitudeMSB);
        currentMemaddress++;
        i2c_eeprom_write_byte(0x50, currentMemaddress, currAltitudeLSB);
        currentMemaddress++;

        if (currAltitude < lastAltitude)
        {
          measures = measures - 1;
          if (measures == 0)
          {
            //fire drogue
            //digitalWrite(pinApogee, HIGH);
            apogeeReadyToFire = true;
            apogeeStartTime = millis();
            //apogeeHasFired=true;

            apogeeAltitude = currAltitude;
          }  
        }
        else 
        {
          lastAltitude = currAltitude;
          measures = 5;
        }   
        if(apogeeReadyToFire)
        {
           if((millis()-apogeeStartTime) > config.outPut1Delay)
           {
             //fire drogue
            digitalWrite(pinApogee, HIGH);
             apogeeReadyToFire = false;
             apogeeHasFired=true;
           }
        }
        if ((currAltitude  < mainDeployAltitude) && apogeeHasFired == true && mainHasFired==false)
        {
          // Deploy main chute  X meters or feet  before landing...
          digitalWrite(pinApogee, LOW);
          mainReadyToFire = true;
            mainStartTime = millis();
          //digitalWrite(pinMain, HIGH); 
          //mainHasFired=true;
          mainAltitude= currAltitude;     
        }
        if(mainReadyToFire)
        {
           if((millis()-mainStartTime) > config.outPut2Delay)
           {
             //fire drogue
            digitalWrite(pinMain, HIGH);
             mainReadyToFire = false;
             mainHasFired=true;
           }
        }
      }
      //end loging
      //store start and end address
      Serial.println(F("stop recording\n "));
      //save end address
      fileA[currentFileNbr].endaddress = currentMemaddress -1;
      saveFlightA(currentFileNbr, startMemaddress, currentMemaddress -1);
      exit =true;
      digitalWrite(pinMain, LOW);
    }
  }
}


//================================================================
// read and write in the microcontroler eeprom
//================================================================
void defaultConfig()
{
  
  config.unit = 0;
  config.beepingMode=0;
  config.outPut1=0;
  config.outPut2=1;
  config.outPut3=2;
  config.outPut1Delay=0;
  config.outPut2Delay=0;
  config.outPut3Delay=0;
  config.mainAltitude=150;
  config.superSonicYesNo=0;
  config.beepingFrequency = 440;
  config.cksum=0xBA;  
}
boolean readAltiConfig() {

  int i;
  for( i=0; i< sizeof(config); i++ ) {
    *((char*)&config + i) = EEPROM.read(CONFIG_START + i);
  }

  if ( config.cksum != 0xBA ) {
    return false;
  }

  return true;

}

int getOutPin(int value)
{

  switch (value)
  {
  case 0:
    pinMain = pyroOut1;
    break;
  case 1:
    pinApogee = pyroOut2;
    break;
  case 2:
    pinOut3 = pyroOut3;
    break; 
  default:

    break;
  }
  return 1;
}
int writeAltiConfig( char *p ) {

  char *str;
  int i=0;
  while ((str = strtok_r(p, ",", &p)) != NULL) // delimiter is the comma
  {
    Serial.println(str);
    switch (i)
    {
    case 1:
      config.unit =atol(str);
      break;
    case 2:
      config.beepingMode=atol(str);
      break;
    case 3:
      config.outPut1=atol(str);
      break;   
    case 4:
      config.outPut2=atol(str);
      break;
    case 5:
      config.outPut3=atol(str);
      break;
    case 6:
      config.mainAltitude=atol(str);
      break;
    case 7:
      config.superSonicYesNo=atol(str);
      break;
    case 8:
      config.outPut1Delay=atol(str);
      break;
    case 9:
      config.outPut2Delay=atol(str);
      break;
    case 10:
      config.outPut3Delay=atol(str);
      break;
    case 11:
      config.beepingFrequency =atol(str);
      break;
    case 12:
      break;
    }
    i++;

  }

  config.cksum = 0xBA;

  for( i=0; i<sizeof(config); i++ ) {
    EEPROM.write(CONFIG_START+i, *((char*)&config + i));
  }
}
//////////////////////////////////////////////////////////////////////////////////////


void printAltiConfig()
{
  readAltiConfig();
  Serial.print(F("$alticonfig"));
  Serial.print(F(","));
  //Unit
  Serial.print(config.unit);
  Serial.print(F(","));
  //beepingMode
  Serial.print(config.beepingMode);
  Serial.print(F(","));
  //output1
  Serial.print(config.outPut1);
  Serial.print(F(","));
  //output2
  Serial.print(config.outPut2);
  Serial.print(F(","));
  //output3
  Serial.print(config.outPut3);
  Serial.print(F(","));
  //supersonicYesNo
  Serial.print(config.superSonicYesNo);
  Serial.print(F(","));
  //mainAltitude
  Serial.print(config.mainAltitude);
  Serial.print(F(","));
  //AltimeterName
  Serial.print(F("AltiMulti"));
  Serial.print(F(","));
  //alti major version
  Serial.print(MAJOR_VERSION);
  //alti minor version
  Serial.print(F(","));
  Serial.print(MINOR_VERSION);
  Serial.print(F(","));
  //output1 delay
  Serial.print(config.outPut1Delay);
  Serial.print(F(","));
  //output2 delay
  Serial.print(config.outPut2Delay);
  Serial.print(F(","));
  //output3 delay
  Serial.print(config.outPut3Delay);
  Serial.print(F(","));
  //Beeping frequency
  Serial.print(config.beepingFrequency);
  Serial.print(F(";\n"));

}

//================================================================
// Main menu to interpret all the commands sent by the altimeter console
//================================================================
void MainMenu()
{
  char readVal=' ';
  int i=0;

  char commandbuffer[100];

  Serial.println(F("Rocket flight data logger. A maximum of 25 flight can be logged \n"));
  Serial.println(F("Commands are: \n"));
  Serial.println(F("w = record flight \n"));
  Serial.println(F("r (followed by the flight number) = read flight data\n"));
  Serial.println(F("l  = print flight list \n"));
  Serial.println(F("e  = erase all flight data \n"));
  Serial.println(F("c  = toggle continuity on/off \n"));
  Serial.println(F("b  = print alti config \n"));
  Serial.println(F("Enter Command and terminate it by a ; >>\n"));
  i=0;
  readVal=' ';
  while( readVal != ';') 
  {
    if (FastReading == false)
    {
      currAltitude = (KalmanCalc(bmp.readAltitude())- initialAltitude);
      if (( currAltitude > liftoffAltitude) != true)
      {
        continuityCheck(pinApogeeContinuity);
        continuityCheck(pinMainContinuity);
        continuityCheck(pinOut3Continuity);
        delay (500);
      }
      else
      {
        recordAltitude();
      }
      if(apogeeHasFired == true && mainHasFired==true)
      {
        beginBeepSeq();

        if(config.beepingMode ==0)
          beepAltitude(apogeeAltitude* FEET_IN_METER);
        else
          beepAltitudeNew(apogeeAltitude * FEET_IN_METER);
        beginBeepSeq();

        if(config.beepingMode ==0)
          beepAltitude(mainAltitude* FEET_IN_METER);
        else
          beepAltitudeNew(mainAltitude* FEET_IN_METER);
      }
    } 
    //if(Serial.available())
    while(Serial.available())
    {
      readVal = Serial.read();
      if (readVal != ';' )
      {
        if (readVal != '\n')
          commandbuffer[i++] = readVal;
      }
      else
      {
        commandbuffer[i++]='\0';
        break;
      }
    }
  }
  Serial.println((char*)commandbuffer);
  if (commandbuffer[0] == 'e')
  {
    Serial.println(F("Erase\n"));
    i2c_eeprom_erase_fileList();
  }  
  else if (commandbuffer[0] == 'r')  
  {
    char temp[3];
    Serial.println(F("Read flight: "));
    Serial.println( commandbuffer[1]);
    Serial.println( "\n");
    temp[0] = commandbuffer[1];
    if (commandbuffer[2] != '\0')
    {
      temp[1] = commandbuffer[2];
      temp[2] = '\0';
    }
    else
      temp[1] ='\0';

    if(atol(temp)>0)
    {
      printFlight(atol(temp));
    }
    else
      Serial.println(F("not a valid flight"));
  }
  // Recording
  else if (commandbuffer[0] == 'w') 
  {
    Serial.println(F("Recording \n"));
    recordAltitude();
  }
  //Number of flight
  else if (commandbuffer[0] == 'n') 
  {
    Serial.println(F("Number of flight \n"));
    Serial.print(F("n;"));
    Serial.println(getFlightList());
    //recordAltitude();
  }
  else if (commandbuffer[0] == 'l')
  {
    Serial.println(F("Flight List: \n"));
    printFlightList();
  }
  else if (commandbuffer[0] == 'c')
  {
    if (noContinuity == false)
    {
      noContinuity = true;
      Serial.println(F("Continuity off \n"));
    }
    else
    {
      noContinuity = false;
      Serial.println(F("Continuity on \n"));
    }
  }
  else if (commandbuffer[0] == 'a')
  {
    Serial.print(F("$start;\n"));
    //getFlightList()
    int i;
    for (i=1; i<getFlightList()+1;i++)
    {
      printFlightData(i);
    }  
    Serial.print(F("$end;\n"));
  }
  //altimeter config
  else if (commandbuffer[0] == 'b')
  {
    Serial.print(F("$start;\n"));

    printAltiConfig();

    Serial.print(F("$end;\n"));
  }
  else if (commandbuffer[0] == 's')
  {
     writeAltiConfig(commandbuffer);
  }
  //FastReading
  else if (commandbuffer[0] == 'f')
  {
    FastReading = true;
    Serial.print(F("$OK;"));

  }
  //FastReading off
  else if (commandbuffer[0] == 'g')
  {
    FastReading = false;
    Serial.print(F("$OK;"));
  }
  else
  {
    Serial.println(F("Unknown command" ));
    Serial.print(F("$UNKNOWN;"));
    Serial.println(commandbuffer[0] + "\n");
  }
}

//================================================================
// beeping functions 
//================================================================

void continuityCheck(int pin)
{
  int val = 0;     // variable to store the read value
  // read the input pin to check the continuity if apogee has not fired
  if (noContinuity == false)
    if (apogeeHasFired == false )
    {
      val = digitalRead(pin);   
      if (val == 0)
      {
        //no continuity long beep
        longBeep();
      }
      else
      {
        //continuity short beep
        shortBeep();
      }
    }
}

void beepAltitude(long altitude)
{
  int i;
  int nbrLongBeep=0;
  int nbrShortBeep=0;
  // this is the last thing that I need to write, some code to beep the altitude
  //altitude is in meters
  //find how many digits
  if(altitude > 99)
  {
    // 1 long beep per hundred meter
    nbrLongBeep= int(altitude /100);
    //then calculate the number of short beep
    nbrShortBeep = (altitude - (nbrLongBeep * 100)) / 10;
  } 
  else
  {
    nbrLongBeep = 0;
    nbrShortBeep = (altitude/10); 
  }
  if (nbrLongBeep > 0)
    for (i = 1; i <  nbrLongBeep +1 ; i++)
    {
      longBeep();
      delay(50);
    } 

  if (nbrShortBeep > 0)
    for (i = 1; i <  nbrShortBeep +1 ; i++)
    {
      shortBeep();
      delay(50);
    } 

  delay(5000);

}

void beginBeepSeq()
{
  int i=0;
  if (NoBeep == false)
  {
    for (i=0; i<10;i++)
    {
      tone(pinSpeaker, 1600,1000);
      delay(50);
      noTone(pinSpeaker);
    }
    delay(1000);
  }
}
void longBeep()
{
  if (NoBeep == false)
  {
    tone(pinSpeaker, config.beepingFrequency,1000);
    delay(1500);
    noTone(pinSpeaker);
  }
}
void shortBeep()
{
  if (NoBeep == false)
  {
    tone(pinSpeaker, config.beepingFrequency,25);
    delay(300);
    noTone(pinSpeaker);
  }
}
void beepAltiVersion (int majorNbr, int minorNbr)
{
  int i;
  for (i=0; i<majorNbr;i++)
  {
    longBeep();
  }
  for (i=0; i<minorNbr;i++)
  {
    shortBeep();
  }  
}
//================================================================
// The following is some Code written by Leo Nutz and modified so that it works
// Output the maximum achieved altitude (apogee) via flashing LED / Buzzer
//================================================================

void longBeepRepeat( int digit ) {
#ifdef DEBUG
  Serial.println("longBeepRepeat: " );
  Serial.println( digit);
#endif
  int i;
  for( i=0; i< digit; i++ ) {
    if( i > 0 ) {
      delay(250);
    }
    longBeep();
  }
}

void shortBeepRepeat( int digit ) {
#ifdef DEBUG
  Serial.println("shortBeepRepeat: " );
  Serial.println( digit);
#endif
  int i;
  for( i=0; i< digit; i++ ) {
    if( i > 0 ) {
      delay(250);
    }
    shortBeep();
  }
}

void beepAltitudeNew(uint32_t value)
{
  char Apogee_String[5];                        // Create an array with a buffer of 5 digits

  ultoa(value, Apogee_String, 10);              // Convert unsigned long to string array, radix 10 for decimal
  uint8_t length = strlen(Apogee_String);       // Get string length

  delay(3000);                                  // Pause for 3 seconds

  for(uint8_t i = 0; i < length; i++ )
  {
    delay(1000);                                // Pause 1 second for every digit output

    uint8_t digit = (Apogee_String[i] - '0');   // Convert ASCI to actual numerical digit
    if ( digit == 0 ) {
      digit = 10;
    }
    if ( digit == 0 ) {
      longBeepRepeat(1);
    } 
    else {
      shortBeepRepeat(digit);
    }
  }
}
//================================================================
// Kalman functions in your code
//================================================================

//Call KalmanInit() once.  

//KalmanInit() - Call before any iterations of KalmanCalc()
void KalmanInit()
{
  kalman_q=4.0001;  //filter parameters, you can play around with them
  kalman_r=.20001;  // but these values appear to be fairly optimal

  kalman_x = 0;
  kalman_p = 0;
  kalman_x_temp = 0;
  kalman_p_temp = 0;

  kalman_x_last = 0;
  kalman_p_last = 0;

}

//KalmanCalc() - Calculates new Kalman values from float value "altitude"
// This will be the ASL altitude during the flight, and the AGL altitude during dumps
float KalmanCalc (float altitude)
{

  //Predict kalman_x_temp, kalman_p_temp
  kalman_x_temp = kalman_x_last;
  kalman_p_temp = kalman_p_last + kalman_r;

  //Update kalman values
  kalman_k = (f_1/(kalman_p_temp + kalman_q)) * kalman_p_temp;
  kalman_x = kalman_x_temp + (kalman_k * (altitude - kalman_x_temp));
  kalman_p = (f_1 - kalman_k) * kalman_p_temp;

  //Save this state for next time
  kalman_x_last = kalman_x;
  kalman_p_last = kalman_p;

  //Assign current Kalman filtered altitude to working variables
  //KAlt = kalman_x; //FLOAT Kalman-filtered altitude value
  return kalman_x;
}  






